#ifndef CODEGENER_SECURE_CHANNEL_ARGS_H
#define CODEGENER_SECURE_CHANNEL_ARGS_H

#include <stdint.h>
#include <stdlib.h>

#include "enclave.h"
/* #include <errno.h> - Errno propagation not enabled so not included. */

/**** User includes. ****/
#include "stdbool.h"
#include "secgear_urts.h"
#ifdef __cplusplus
extern "C" {
#endif

/**** User defined types in EDL. ****/
/* There were no user defined types. */

/**** Trusted function marshalling structs. ****/
typedef struct _get_enclave_pubkey_size_t
{
    size_t retval_size;
    size_t session_id_size;
    size_t pubkey_size;
    size_t pubkey_len_size;
} get_enclave_pubkey_size_t;

typedef struct _init_session_size_t
{
    size_t retval_size;
    size_t session_id_size;
} init_session_size_t;

typedef struct _set_enc_key_size_t
{
    size_t retval_size;
    size_t session_id_size;
    size_t data_size;
    size_t data_len_size;
} set_enc_key_size_t;

typedef struct _get_enclave_exch_param_len_size_t
{
    size_t retval_size;
    size_t session_id_size;
    size_t exch_param_len_size;
} get_enclave_exch_param_len_size_t;

typedef struct _get_enclave_exch_param_size_t
{
    size_t retval_size;
    size_t session_id_size;
    size_t exch_param_size;
    size_t exch_param_len_size;
} get_enclave_exch_param_size_t;

typedef struct _set_peer_exch_param_size_t
{
    size_t retval_size;
    size_t session_id_size;
    size_t data_size;
    size_t data_len_size;
} set_peer_exch_param_size_t;

typedef struct _del_enclave_sec_chl_size_t
{
    size_t retval_size;
    size_t session_id_size;
} del_enclave_sec_chl_size_t;

typedef struct _enclave_start_sec_chl_size_t
{
    size_t retval_size;
} enclave_start_sec_chl_size_t;

typedef struct _enclave_stop_sec_chl_size_t
{
    size_t retval_size;
} enclave_stop_sec_chl_size_t;

typedef struct _enclave_check_session_timeout_size_t
{
    size_t retval_size;
} enclave_check_session_timeout_size_t;

/**** Untrusted function marshalling structs. ****/

/**** Trusted function IDs ****/
enum
{
    fid_get_enclave_pubkey = 2,
    fid_init_session = 3,
    fid_set_enc_key = 4,
    fid_get_enclave_exch_param_len = 5,
    fid_get_enclave_exch_param = 6,
    fid_set_peer_exch_param = 7,
    fid_del_enclave_sec_chl = 8,
    fid_enclave_start_sec_chl = 9,
    fid_enclave_stop_sec_chl = 10,
    fid_enclave_check_session_timeout = 11,
    fid_trusted_call_id_max = SECGEAR_ENUM_MAX
};

/**** Trusted switchless function IDs ****/
enum
{

    fid_trusted_switchless_call_id_max = SECGEAR_ENUM_MAX
};
/**** Untrusted function IDs ****/
enum
{

    fid_untrusted_call_id_max = SECGEAR_ENUM_MAX
};

#ifdef __cplusplus
}
#endif

#endif
